@@ -36,9 +36,7 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     rlreg <- function(xcoef,lpoutlier,tau2,x,y,offset,prop) {
       poutlier <- 1.0/(1+exp(-lpoutlier))
       
-      lp <- as.vector(x %*% xcoef)
-      
-      if(!is.null(offset)) lp <- lp+offset
+      lp <- as.vector(x %*% xcoef)+offset
       
       lp1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)
       lp2 <- llrandbinomcpp(y, lp, tau2, gh)
@@ -60,87 +58,75 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       return(rlreg(matrix(p[1:(length(p)-1)],ncol=1),lpoutlier,p[length(p)],x,y,offset,prop))
     }
     
-    thedata <- data.frame(y,x)
-
-    lastdata=length(names(thedata))
-    if(!is.null(offset)) thedata <- data.frame(thedata,offset=offset)
-     
-    # fit first with glm
-    if (is.null(starting.values)) {
+    tryCatch({
+      if (is.null(starting.values)) {
+        robust.binomial.prefit <- glm(cbind(y[,1],y[,2])~x[,colnames(x)!="(Intercept)"],family=binomial(), offset=offset,
+                                      subset=(outliers!=1))
+        
+        
+        # get the starting values
+        prefit.coef <- coef(robust.binomial.prefit)
+        # assume 20% outliers as a starting point
+        currlpoutlier <- log(0.2/(1-0.2))
+        currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
+        currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
+        currtau2 <- min(rgamma(1,2,1),2)
+      } else
+      {
+        currxcoef <- matrix(starting.values[1:(length(starting.values)-2)],ncol=1)
+        currlpoutlier <- starting.values[length(starting.values)-1]
+        currtau2 <- starting.values[length(starting.values)]
+      }
       
-    if (lastdata>3) theformula <- paste(names(thedata)[4:lastdata],"+",sep='',collapse="")
-    else theformula <- "1+"
-    
-    theformula <- substr(theformula,1,nchar(theformula)-1)
-    
-    theformula <- paste("cbind(",names(thedata)[1],",",names(thedata)[2],")~",theformula,sep='',collapse="")
-    
-    doprefit <- paste("robust.binomial.prefit <- glm(",theformula,",",
-                "family=binomial(), offset=offset,subset=(outliers!=1),data=thedata)",sep='',collapse='')
-    
-    eval(parse(text=doprefit))
-    
-    # get the starting values
-    prefit.coef <- coef(robust.binomial.prefit)
-    # assume 20% outliers as a starting point
-    currlpoutlier <- log(0.2/(1-0.2))
-    currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
-    currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
-    currtau2 <- min(rgamma(1,2,1),2)
-    } else
-    {
-      currxcoef <- matrix(starting.values[1:(length(starting.values)-2)],ncol=1)
-       currlpoutlier <- starting.values[length(starting.values)-1]
-      currtau2 <- starting.values[length(starting.values)]
-    }
-    
-    currll <- -1.0e100
-    nem <- 0
-    
-    repeat {
-      nem <- nem+1
-      # expectation step
-      currpoutlier <- 1.0/(1+exp(-currlpoutlier))
+      currll <- -1.0e100
+      nem <- 0
       
-      lp <- as.vector(x %*% currxcoef)
-      
-      if (!is.null(offset)) lp <- lp+offset
-      
-      ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-currpoutlier) 
-      ll2 <- llrandbinomcpp(y, lp, currtau2, gh)+log(currpoutlier)
-      
-      ll <- cbind(ll1,ll2)
-      prop <- t(apply(ll,1,function(x) {
-        x <- x-max(x)
-        x <- ifelse(x==-Inf,-1e100,x)
-        return(exp(x)/sum(exp(x)))
-      }))
-      # calculate outlier proportion
-      poutlier <- sum(prop[,2])/dim(prop)[1]
-      currlpoutlier <- log(poutlier/(1-poutlier))
-      
-      if (is.na(poutlier)) stop("poutlier is NA")
-      
-      startvals <- c(currxcoef,currtau2)
-      names(startvals) <- c(dimnames(x)[[2]],"tau2")
-      
-      results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
-                            lower=c(rep(-Inf,length(currxcoef)),0),
-                            #control=list(trace=1,iter.max=10),
-                            control=list(iter.max=5),
-                            lpoutlier=currlpoutlier,
-                            prop=prop,
-                            y=y,x=x,offset=offset,
-                            fixed=fixed))
-      currxcoef <- matrix(as.numeric(results.nlm$par)[1:(length(results.nlm$par)-1)],ncol=1)
-      currtau2 <- as.numeric(results.nlm$par)[length(results.nlm$par)]
-      
-      lastll <- currll
-      currll <- -rlreg(currxcoef,currlpoutlier,currtau2,x,y,offset)
-      if (abs((lastll-currll)/currll)<EMTol) break()
-      if (nem >100) break()
-    }
-    return(list(ll=currll,start.val=c(currxcoef,currlpoutlier,currtau2)))    
+      repeat {
+        nem <- nem+1
+        # expectation step
+        currpoutlier <- 1.0/(1+exp(-currlpoutlier))
+        
+        lp <- as.vector(x %*% currxcoef)+offset
+        
+        ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-currpoutlier) 
+        ll2 <- llrandbinomcpp(y, lp, currtau2, gh)+log(currpoutlier)
+        
+        ll <- cbind(ll1,ll2)
+        prop <- t(apply(ll,1,function(x) {
+          x <- x-max(x)
+          x <- ifelse(x==-Inf,-1e100,x)
+          return(exp(x)/sum(exp(x)))
+        }))
+        # calculate outlier proportion
+        poutlier <- sum(prop[,2])/dim(prop)[1]
+        currlpoutlier <- log(poutlier/(1-poutlier))
+        
+        if (is.na(poutlier)) stop("poutlier is NA")
+        
+        startvals <- c(currxcoef,currtau2)
+        names(startvals) <- c(dimnames(x)[[2]],"tau2")
+        
+        results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
+                                               lower=c(rep(-Inf,length(currxcoef)),0),
+                                               #control=list(trace=1,iter.max=10),
+                                               control=list(iter.max=5),
+                                               lpoutlier=currlpoutlier,
+                                               prop=prop,
+                                               y=y,x=x,offset=offset,
+                                               fixed=fixed))
+        currxcoef <- matrix(as.numeric(results.nlm$par)[1:(length(results.nlm$par)-1)],ncol=1)
+        currtau2 <- as.numeric(results.nlm$par)[length(results.nlm$par)]
+        
+        lastll <- currll
+        currll <- -rlreg(currxcoef,currlpoutlier,currtau2,x,y,offset)
+        if (verbose) print(sprintf("Likelihood at end of EM step %.4f", currll))
+        if (abs((lastll-currll)/currll)<EMTol) break()
+        if (nem >100) break()
+      }
+      return(list(ll=currll,start.val=c(currxcoef,currlpoutlier,currtau2)))    
+    },
+    error=function(e) return(list(ll=NA))
+    )
   }
   
   ll.robustbinomial <- function(p){
@@ -151,9 +137,7 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     
     poutlier <- exp(lpoutlier)/(1+exp(lpoutlier))
     
-    lp <- as.vector(x %*% xcoef)
-    
-    if (!is.null(offset)) lp <- lp+offset
+    lp <- as.vector(x %*% xcoef)+offset
     
     ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-poutlier)
     
@@ -165,8 +149,8 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     if (!is.finite(negll)) negll <- NA
     return(negll)
   }
-    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
-    seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
+  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
+  seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
   
   if (is.null(starting.values)) {
     if (cores > 1) {
@@ -179,27 +163,36 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
                       fitonemlreg(y,outliers,x,offset,fixed=NULL)}
       parallel::stopCluster(cl)
       maxll <- -Inf
+      nfails <- 0
       for (i in 1:notrials) {
-        if (verbose) print(c(res[[i]]$ll,res[[i]]$start.val))
-        if (res[[i]]$ll>maxll) {
-          maxll <- res[[i]]$ll
-          start.val <- res[[i]]$start.val
+        if (verbose) cat(c(res[[i]]$ll,res[[i]]$start.val),"\n")
+        if (is.na(res[[i]]$ll)) nfails <- nfails+1
+        else {
+          if (res[[i]]$ll>maxll) {
+            maxll <- res[[i]]$ll
+            start.val <- res[[i]]$start.val
+          }
         }
+        if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
       }
     } else {
       maxll <- -Inf
+      nfails <- 0
       for (i in 1:notrials) {
-        try({
-          noutliers <- max(1,round(dim(x)[1]*0.2))
-          outliers <- sample(c(rep(1,noutliers),rep(0,dim(x)[1]-noutliers)),dim(x)[1])
-          thefit <- fitonemlreg(y,outliers,x,offset,fixed=NULL)
-          if (verbose) print(c(thefit$ll,thefit$start.val))
+        noutliers <- max(1,round(dim(x)[1]*0.2))
+        outliers <- sample(c(rep(1,noutliers),rep(0,dim(x)[1]-noutliers)),dim(x)[1])
+        if (verbose) print(sprintf("Trial %i", i))
+        thefit <- fitonemlreg(y,outliers,x,offset,fixed=NULL)
+        if (verbose) print(sprintf("Likelihood for trial %.4f", thefit$ll))
+        if (verbose) print(thefit$start.val)
+        if (is.na(thefit$ll)) nfails <- nfails+1
+        else {
           if (thefit$ll>maxll) {
             maxll <- thefit$ll
             start.val <- thefit$start.val
           }
         }
-        )
+        if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
       }
     }
   } else {
@@ -215,13 +208,13 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
   lower.val <- c(rep(-Inf,length(start.val)-2),-Inf,0)
   
   names(lower.val) <- names(start.val)
-
+  
   robustbinomial.fit <- mle2(ll.robustbinomial,start=start.val,vecpar=TRUE,
                              optimizer="user",optimfun=myoptim,
                              data=list(y=y,x=x,offset=offset),
-                            skip.hessian=TRUE,trace=verbose,
-                            lower=lower.val,
-                            control=if (verbose) list(eval.max=1000,iter.max=1000,trace=5) else list(eval.max=1000,iter.max=1000))
+                             skip.hessian=TRUE,trace=verbose,
+                             lower=lower.val,
+                             control=if (verbose) list(eval.max=1000,iter.max=1000,trace=5) else list(eval.max=1000,iter.max=1000))
   if (calcHessian) {
     thecoef <- coef(robustbinomial.fit)
     ncoef <- length(thecoef)
@@ -235,10 +228,8 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
   lpoutlier <- coef(robustbinomial.fit)[length(coef(robustbinomial.fit))-1]
   poutlier <- 1.0/(1+exp(-lpoutlier))
   tau2 <- coef(robustbinomial.fit)[length(coef(robustbinomial.fit))]
- 
-  lp <- as.vector(x %*% xcoef)
   
-  if (!is.null(offset)) lp <- lp+offset
+  lp <- as.vector(x %*% xcoef)+offset
   
   ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-poutlier) 
   ll2 <- llrandbinomcpp(y, lp, tau2, gh)+log(poutlier)
