@@ -7,18 +7,18 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     rlreg <- function(xcoef,lpoutlier,tau2, sigma2,x,y,offset,prop) {
       poutlier <- 1.0/(1+exp(-lpoutlier))
       
-      lp <- as.vector(x %*% xcoef)
-      
-      if(!is.null(offset)) lp <- lp+offset
+      lp <- as.vector(x %*% xcoef)+offset
       
       lp1 <- dnorm(y, mean=lp, sd=sqrt(sigma2), log = TRUE)
       lp2 <- dnorm(y, mean=lp, sd=sqrt(tau2+sigma2), log=TRUE)
       
-      if (!missing(prop))  ll <- prop*cbind(lp1,lp2)
-      else ll <- cbind(lp1+log(1-poutlier),lp2+log(poutlier))
-      
-      maxll <- apply(ll,1,max)
-      negll <- -sum(maxll+log(apply(exp(ll-maxll),1,sum)))
+      if (!missing(prop)) {
+        ll <- prop*cbind(lp1,lp2)
+        negll <- -sum(apply(ll,1,sum))
+      } else {
+        l <- exp(cbind(lp1+log(1-poutlier),lp2+log(poutlier)))
+        negll <- -sum(log(apply(l,1,sum)))
+      }
       if (is.nan(negll)) negll <- NA
       if (!is.finite(negll)) negll <- NA
       return(negll)
@@ -28,29 +28,12 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       p[names(fixed)] <- fixed
       return(rlreg(matrix(p[1:(length(p)-2)],ncol=1),lpoutlier,p[length(p)-1],p[length(p)],x,y,offset,prop))
     }
-    
-    thedata <- data.frame(y,x)
-    
-    lastdata=length(names(thedata))
-    if(!is.null(offset)) thedata <- data.frame(thedata,offset=offset)
-    
-    # get the starting values
-    if (is.null(starting.values)) {
-      # fit first with glm
-      
-      if (lastdata>2) theformula <- paste(names(thedata)[3:lastdata],"+",sep='',collapse="")
-      else theformula <- "1+"
-      
-      theformula <- substr(theformula,1,nchar(theformula)-1)
-      
-      
-      doprefit <- paste("robust.gaussian.prefit <- lm(y~",theformula,",",
-                        "offset=offset,subset=(outliers!=1),
-                   data=thedata)",sep='',collapse='')
-      eval(parse(text=doprefit))
-      
+          
+    tryCatch({
+      if (is.null(starting.values)) {
+      robust.gaussian.prefit <- lm(y~x[,colnames(x)!="(Intercept)"],offset=offset,subset=(outliers!=1))
       prefit.coef <- coef(robust.gaussian.prefit)
-      # assume 50% outliers as a starting point
+      # assume 20% outliers as a starting point
       currlpoutlier <- log(0.2/(1-0.2))
       currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
       currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
@@ -70,9 +53,7 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       # expectation step
       currpoutlier <- 1.0/(1+exp(-currlpoutlier))
       
-      lp <- as.vector(x %*% currxcoef)
-      
-      if (!is.null(offset)) lp <- lp+offset
+      lp <- as.vector(x %*% currxcoef)+offset
       
       ll1 <- dnorm(y, mean=lp, sd=sqrt(currsigma2), log = TRUE)+log(1-currpoutlier) 
       ll2 <- dnorm(y, mean=lp, sd=sqrt(currsigma2+currtau2),log=TRUE)+log(currpoutlier)
@@ -91,8 +72,7 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       
       startvals <- c(currxcoef,currtau2,currsigma2)
       names(startvals) <- c(dimnames(x)[[2]],"tau2","sigma2")
-      
-      results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
+       results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
                                              lower=c(rep(-Inf,length(currxcoef)),rep(0,2)),
                                              #control=list(trace=1,iter.max=10),
                                              control=list(trace=0,iter.max=5),
@@ -100,16 +80,21 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
                                              prop=prop,
                                              y=y,x=x,offset=offset,
                                              fixed=fixed))
+   
       currxcoef <- matrix(as.numeric(results.nlm$par)[1:(length(results.nlm$par)-2)],ncol=1)
       currtau2 <- as.numeric(results.nlm$par)[length(results.nlm$par)-1]
       currsigma2 <- as.numeric(results.nlm$par)[length(results.nlm$par)]
       
       lastll <- currll
       currll <- -rlreg(currxcoef,currlpoutlier,currtau2,currsigma2,x,y,offset)
-      if (abs((lastll-currll)/currll)<EMTol) break()
+      if (verbose) print(sprintf("Likelihood at end of EM step %.4f", currll))
+       if (abs((lastll-currll)/currll)<EMTol) break()
       if (nem >100) break()
     }
     return(list(ll=currll,start.val=c(currxcoef,currlpoutlier,currtau2,currsigma2)))    
+    },
+    error=function(e) return(list(ll=NA))
+    )
   }
   
   ll.robustgaussian <- function(p){
@@ -121,9 +106,7 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     
     poutlier <- 1.0/(1+exp(-lpoutlier))
     
-    lp <- as.vector(x %*% xcoef)
-    
-    if (!is.null(offset)) lp <- lp+offset
+    lp <- as.vector(x %*% xcoef)+offset
     
     ll1 <- dnorm(y, mean=lp, sd=sqrt(sigma2), log = TRUE)+log(1-poutlier)
     
@@ -152,29 +135,38 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       parallel::stopCluster(cl)
 #      browser()
       maxll <- -Inf
+      nfails <- 0
       for (i in 1:notrials) {
         if (verbose) cat(c(res[[i]]$ll,res[[i]]$start.val),"\n")
-        if (res[[i]]$ll>maxll) {
-          maxll <- res[[i]]$ll
-          start.val <- res[[i]]$start.val
+        if (is.na(res[[i]]$ll)) nfails <- nfails+1
+        else {
+          if (res[[i]]$ll>maxll) {
+            maxll <- res[[i]]$ll
+            start.val <- res[[i]]$start.val
+          }
         }
+        if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
       }
     } else {
       maxll <- -Inf
+      nfails <- 0
       for (i in 1:notrials) {
-        try({
           noutliers <- max(1,round(dim(x)[1]*0.2))
           outliers <- sample(c(rep(1,noutliers),rep(0,dim(x)[1]-noutliers)),dim(x)[1])
+          if (verbose) print(sprintf("Trial %i", i))
           thefit <- fitonemlreg(y,outliers,x,offset,fixed=NULL)
-          if (verbose) print(c(thefit$ll,thefit$start.val))
-          if (thefit$ll>maxll) {
-            maxll <- thefit$ll
-            start.val <- thefit$start.val
+          if (verbose) print(sprintf("Likelihood for trial %.4f", thefit$ll))
+          if (verbose) print(thefit$start.val)
+          if (is.na(thefit$ll)) nfails <- nfails+1
+          else {
+            if (thefit$ll>maxll) {
+              maxll <- thefit$ll
+              start.val <- thefit$start.val
+            }
           }
+          if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
         }
-        )
-      }
-    }
+     }
   } else {
     start.val <- starting.values
   }
@@ -213,9 +205,7 @@ gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
   sigma2 <- coef(robustgaussian.fit)[length(coef(robustgaussian.fit))]
   
   
-  lp <- as.vector(x %*% xcoef)
-  
-  if (!is.null(offset)) lp <- lp+offset
+  lp <- as.vector(x %*% xcoef)+offset
   
   ll1 <- dnorm(y, mean=lp, sd=sqrt(sigma2),log = TRUE)+log(1-poutlier) 
   ll2 <- dnorm(y, mean=lp, sd=sqrt(sigma2+tau2),log = TRUE)+log(poutlier)
